// Copyright (c) 2020, DartRocket authors.
// MIT-style license that can be found in the LICENSE file.

library dartrocket_container;

import 'src/binding.dart';

class Container {
  /// Bindings for both factories and singletons.
  var _bindings = Map<String, Binding>();
  /// Aliases for shortening namespaces of bindings.
  var _aliases = Map<String, String>();

  /// Function to create a lazy load factory binding.
  void bind(String namespace, Function closure) {
    this._bindings[namespace] = Binding(closure);
  }

  /// Function to create a lazy load singleton binding.
  void singleton(String namespace, Function closure) {
    this._bindings[namespace] = Binding(closure, singleton: true);
  }

  /// Function to map a alias to a namespace.
  void alias(String namespace, String alias) {
    this._aliases[alias] = namespace;
  }

  /// Function to get an instance of the binding.
  dynamic use(String namespace) {
    return this._bindings[this._aliases[namespace]]?.instance()
      ?? this._bindings[namespace]?.instance();
  }
}